@@ -14,7 +14,7 @@ use crate::{
     Version,
 };
 use serde::{de, ser, Deserialize, Serialize};
-use std::{fmt, str::FromStr};
+use std::{collections::HashMap, collections::HashSet, fmt, fmt::Write, str::FromStr};
 
 impl<'de> Deserialize<'de> for Lockfile {
     fn deserialize<D: de::Deserializer<'de>>(
@@ -113,6 +113,18 @@ impl From<&Lockfile> for EncodableLockfile {
         let mut packages = Vec::with_capacity(lockfile.packages.len());
         let mut metadata = lockfile.metadata.clone();
 
+        let mut package_to_registries: HashMap<_, HashSet<&SourceId>> = HashMap::new();
+        for package in &lockfile.packages {
+            if let Some(source) = package.source.as_ref() {
+                if source.is_registry() {
+                    package_to_registries
+                        .entry(&package.name)
+                        .or_default()
+                        .insert(source);
+                }
+            }
+        }
+
         for package in &lockfile.packages {
             let mut raw_pkg = EncodablePackage::from(package);
             let checksum_key = metadata::MetadataKey::for_checksum(&Dependency::from(package));
@@ -133,6 +145,16 @@ impl From<&Lockfile> for EncodableLockfile {
                 // metadata table if present
                 raw_pkg.v2_deps(&lockfile.packages);
                 metadata.remove(&checksum_key);
+
+                // If there's only one registry, Cargo does not encode this information so
+                // it should be omitted from encodable packages
+                for dep in raw_pkg.dependencies.iter_mut() {
+                    if let Some(registries) = package_to_registries.get(&dep.name) {
+                        if registries.len() == 1 {
+                            dep.source = None;
+                        }
+                    }
+                }
             }
 
             packages.push(raw_pkg);
@@ -173,7 +195,7 @@ impl ToString for EncodableLockfile {
         if let Some(value) = toml.get("version") {
             if let Some(version) = value.as_integer() {
                 if version >= 3 {
-                    out.push_str(&format!("version = {}\n", version));
+                    writeln!(out, "version = {}", version).unwrap();
                 }
             }
         }
@@ -199,7 +221,7 @@ impl ToString for EncodableLockfile {
 
         if let Some(meta) = toml.get("metadata") {
             out.push_str("[metadata]\n");
-            out.push_str(&meta.to_string());
+            out.push_str(&toml::to_string_pretty(&meta).unwrap());
         }
 
         // Trim redundant newlines
@@ -216,14 +238,14 @@ impl ToString for EncodableLockfile {
 /// This method is adapted from the same-named method in upstream Cargo:
 /// <https://github.com/rust-lang/cargo/blob/0c70319/src/cargo/ops/lockfile.rs#L194-L221>
 fn emit_package(dep: &toml::value::Table, out: &mut String) {
-    out.push_str(&format!("name = {}\n", &dep["name"]));
-    out.push_str(&format!("version = {}\n", &dep["version"]));
+    writeln!(out, "name = {}", &dep["name"]).unwrap();
+    writeln!(out, "version = {}", &dep["version"]).unwrap();
 
     if dep.contains_key("source") {
-        out.push_str(&format!("source = {}\n", &dep["source"]));
+        writeln!(out, "source = {}", &dep["source"]).unwrap();
     }
     if dep.contains_key("checksum") {
-        out.push_str(&format!("checksum = {}\n", &dep["checksum"]));
+        writeln!(out, "checksum = {}", &dep["checksum"]).unwrap();
     }
 
     if let Some(s) = dep.get("dependencies") {
@@ -233,13 +255,13 @@ fn emit_package(dep: &toml::value::Table, out: &mut String) {
             out.push_str("dependencies = [\n");
 
             for child in slice.iter() {
-                out.push_str(&format!(" {},\n", child));
+                writeln!(out, " {},", child).unwrap();
             }
 
             out.push_str("]\n");
         }
     } else if dep.contains_key("replace") {
-        out.push_str(&format!("replace = {}\n", &dep["replace"]));
+        writeln!(out, "replace = {}", &dep["replace"]).unwrap();
     }
 
     out.push('\n');
