@@ -17,6 +17,8 @@ use std::path::Path;
 
 /// Location of the crates.io index
 pub const CRATES_IO_INDEX: &str = "https://github.com/rust-lang/crates.io-index";
+/// Location of the crates.io sparse HTTP index
+pub const CRATES_IO_SPARSE_INDEX: &str = "sparse+https://index.crates.io/";
 
 /// Default branch name
 pub const DEFAULT_BRANCH: &str = "master";
@@ -39,6 +41,7 @@ pub struct SourceId {
 
 /// The possible kinds of code source.
 #[derive(Clone, Debug, Eq, Hash, PartialEq, PartialOrd, Ord)]
+#[non_exhaustive]
 pub enum SourceKind {
     /// A git repository.
     Git(GitReference),
@@ -49,6 +52,9 @@ pub enum SourceKind {
     /// A remote registry.
     Registry,
 
+    /// A sparse registry.
+    SparseRegistry,
+
     /// A local filesystem-based registry.
     LocalRegistry,
 
@@ -109,9 +115,14 @@ impl SourceId {
                 Ok(SourceId::new(SourceKind::Registry, url)?
                     .with_precise(Some("locked".to_string())))
             }
+            "sparse" => {
+                let url = url.into_url()?;
+                Ok(SourceId::new(SourceKind::SparseRegistry, url)?
+                    .with_precise(Some("locked".to_string())))
+            }
             "path" => Self::new(SourceKind::Path, url.into_url()?),
             kind => Err(Error::Parse(format!(
-                "unsupported source protocol: {}",
+                "unsupported source protocol: `{}` from `{string}`",
                 kind
             ))),
         }
@@ -130,7 +141,7 @@ impl SourceId {
         Self::new(SourceKind::Git(reference), url.clone())
     }
 
-    /// Creates a SourceId from a registry URL.
+    /// Creates a SourceId from a remote registry URL.
     pub fn for_registry(url: &Url) -> Result<Self> {
         Self::new(SourceKind::Registry, url.clone())
     }
@@ -184,7 +195,10 @@ impl SourceId {
 
     /// Returns `true` if this source is from a registry (either local or not).
     pub fn is_registry(&self) -> bool {
-        matches!(self.kind, SourceKind::Registry | SourceKind::LocalRegistry)
+        matches!(
+            self.kind,
+            SourceKind::Registry | SourceKind::SparseRegistry | SourceKind::LocalRegistry
+        )
     }
 
     /// Returns `true` if this source is a "remote" registry.
@@ -192,7 +206,7 @@ impl SourceId {
     /// "remote" may also mean a file URL to a git index, so it is not
     /// necessarily "remote". This just means it is not `local-registry`.
     pub fn is_remote_registry(&self) -> bool {
-        matches!(self.kind, SourceKind::Registry)
+        matches!(self.kind, SourceKind::Registry | SourceKind::SparseRegistry)
     }
 
     /// Returns `true` if this source from a Git repository.
@@ -225,6 +239,8 @@ impl SourceId {
     /// Returns `true` if the remote registry is the standard <https://crates.io>.
     pub fn is_default_registry(&self) -> bool {
         self.kind == SourceKind::Registry && self.url.as_str() == CRATES_IO_INDEX
+            || self.kind == SourceKind::SparseRegistry
+                && self.url.as_str() == &CRATES_IO_SPARSE_INDEX[7..]
     }
 }
 
@@ -270,6 +286,11 @@ impl fmt::Display for SourceId {
                 ref url,
                 ..
             } => write!(f, "registry+{}", url),
+            SourceId {
+                kind: SourceKind::SparseRegistry,
+                ref url,
+                ..
+            } => write!(f, "sparse+{}", url),
             SourceId {
                 kind: SourceKind::LocalRegistry,
                 ref url,
@@ -366,7 +387,10 @@ mod tests {
     use super::SourceId;
 
     #[test]
-    fn default_works() {
-        SourceId::default();
+    fn identifies_crates_io() {
+        assert!(SourceId::default().is_default_registry());
+        assert!(SourceId::from_url(super::CRATES_IO_SPARSE_INDEX)
+            .expect("failed to parse sparse URL")
+            .is_default_registry());
     }
 }
